@@ -2,7 +2,7 @@
 #define roger_H
 
 void sampleNRM(double *theta, double *b, int *a, int *i, int *first, int *last, int *m, int *response);
-void sampleNRM2(double *theta, double *b, int *a, int *first, int *last, int *nI, int *m, int *score);
+void sampleNRM2(double *theta, double *b, int *a, int *first, int *last, int *nI, int *m, int *maxA, int *score);
 //void PV1(double *b, int *a, int *first, int *last, double *mu, double *sigma, int *score, int *pop, int *nP,int *nI, int *nPop, double *theta);
 void PV(double *b, int *a, int *first, int *last, double *mu, double *sigma, int *score, int *pop, int *nP,int *nI, int *nPV, double *theta);
 void PVMix(double *b, int *a, int *first, int *last, double *alpha, double *mu, double *sigma, int *score, int *nP, int *nI, int *nPV, double *theta);
